;;; custom-load.el --- automatically extracted custom dependencies

;;; Code:

(autoload 'custom-add-loads "cus-load")

(custom-add-loads 'extensions '("auto-show" "wid-edit"))
(custom-add-loads 'info-faces '("info"))
(custom-add-loads 'custom-buffer '("cus-edit"))
(custom-add-loads 'custom-faces '("cus-edit"))
(custom-add-loads 'auto-show '("auto-show"))
(custom-add-loads 'drag-n-drop '("dragdrop"))
(custom-add-loads 'mouse '("mouse" "mwheel"))
(custom-add-loads 'etags '("etags"))
(custom-add-loads 'package-tools '("package-get" "package-ui"))
(custom-add-loads 'widgets '("wid-browse" "wid-edit"))
(custom-add-loads 'menu '("menubar-items"))
(custom-add-loads 'minibuffer '("simple" "minibuf"))
(custom-add-loads 'log-message '("simple"))
(custom-add-loads 'environment '("frame" "keydefs" "minibuf" "modeline" "window-xemacs" "gutter" "cus-edit" "gnuserv" "menubar" "sound" "toolbar-items" "x-init"))
(custom-add-loads 'sound '("sound"))
(custom-add-loads 'pui '("package-ui"))
(custom-add-loads 'terminals '("gnuserv"))
(custom-add-loads 'auto-save '("files" "auto-save"))
(custom-add-loads 'mail '("simple" "startup"))
(custom-add-loads 'custom-menu '("cus-edit"))
(custom-add-loads 'docs '("hyper-apropos" "info"))
(custom-add-loads 'tools '("etags" "hyper-apropos"))
(custom-add-loads 'editing-basics '("cmdloop" "simple" "files" "lisp"))
(custom-add-loads 'internal '("cus-edit"))
(custom-add-loads 'help-appearance '("help" "help-macro"))
(custom-add-loads 'build-report '("build-report"))
(custom-add-loads 'buffers-menu '("menubar-items"))
(custom-add-loads 'hypermedia '("wid-edit"))
(custom-add-loads 'lisp '("lisp" "lisp-mode" "hyper-apropos"))
(custom-add-loads 'applications '("cus-edit"))
(custom-add-loads 'help '("help" "cus-edit" "hyper-apropos" "info"))
(custom-add-loads 'keyboard '("cmdloop"))
(custom-add-loads 'hyper-apropos-faces '("hyper-apropos"))
(custom-add-loads 'ldap '("ldap"))
(custom-add-loads 'widget-browse '("wid-browse"))
(custom-add-loads 'data '("auto-save"))
(custom-add-loads 'warnings '("simple"))
(custom-add-loads 'widget-documentation '("wid-edit"))
(custom-add-loads 'comm '("ldap"))
(custom-add-loads 'backup '("files"))
(custom-add-loads 'frames '("frame" "window-xemacs" "gnuserv" "gui"))
(custom-add-loads 'customize '("wid-edit" "cus-edit"))
(custom-add-loads 'custom-browse '("cus-edit"))
(custom-add-loads 'abbrev '("abbrev" "files"))
(custom-add-loads 'programming '("cus-edit"))
(custom-add-loads 'printing '("printer"))
(custom-add-loads 'toolbar '("toolbar-items"))
(custom-add-loads 'dired '("files"))
(custom-add-loads 'dnd-debug '("dragdrop"))
(custom-add-loads 'package-get '("package-get"))
(custom-add-loads 'killing '("simple"))
(custom-add-loads 'widget-button '("wid-edit"))
(custom-add-loads 'paren-blinking '("simple"))
(custom-add-loads 'find-file '("files"))
(custom-add-loads 'font-menu '("font-menu"))
(custom-add-loads 'files '("files"))
(custom-add-loads 'build '("build-report"))
(custom-add-loads 'font-lock '("font-lock"))
(custom-add-loads 'external '("process" "cus-edit"))
(custom-add-loads 'development '("process" "lisp-mode" "cus-edit"))
(custom-add-loads 'gnuserv '("gnuserv"))
(custom-add-loads 'gutter '("gutter" "gutter-items"))
(custom-add-loads 'fill-comments '("simple"))
(custom-add-loads 'windows '("window" "window-xemacs"))
(custom-add-loads 'widget-faces '("wid-edit"))
(custom-add-loads 'languages '("lisp-mode" "cus-edit" "font-lock"))
(custom-add-loads 'fill '("simple" "fill"))
(custom-add-loads 'custom-magic-faces '("cus-edit"))
(custom-add-loads 'display '("modeline" "auto-show" "scrollbar" "toolbar"))
(custom-add-loads 'faces '("faces" "cus-edit" "font-lock" "font" "hyper-apropos" "info" "wid-edit"))
(custom-add-loads 'emacs '("faces" "help" "files" "cus-edit" "package-get"))
(custom-add-loads 'processes '("process" "gnuserv"))
(custom-add-loads 'hyper-apropos '("hyper-apropos"))
(custom-add-loads 'wp '("printer" "cus-edit"))
(custom-add-loads 'vc '("files"))
(custom-add-loads 'isearch '("isearch-mode"))
(custom-add-loads 'font-lock-faces '("font-lock"))
(custom-add-loads 'modeline '("modeline"))
(custom-add-loads 'editing '("simple" "abbrev" "fill" "cus-edit" "dragdrop" "mouse"))
(custom-add-loads 'matching '("simple" "isearch-mode" "hyper-apropos"))
(custom-add-loads 'i18n '("cus-edit"))
(custom-add-loads 'info '("info" "toolbar-items"))
(custom-add-loads 'x '("gtk-faces" "x-faces"))
(custom-add-loads 'buffers-tab '("gutter-items"))

;;; custom-load.el ends here
